'''
This module provides abstract bases for Command like classes.
'''

import requests


def everyparser(parser, argbloks):
    'intention of this function is to be called from every subparser function'
    argbloks.default_account_argument(parser)
    argbloks.default_config_argument(parser)
    argbloks.optional_debug_flag(parser)
    return parser


class CommandBase:
    '''
    CommandBase serves as base class for simple commands.
    It provides creation context handling, and context merging.

    You should provide __call__ method that implements actual action.
    '''

    authorization_capability = None

    def __init__(self, creation_context):
        '''
        Creates base command. It takes creation_context, which should be
        dictionary of preconfiguration for this command.
        '''
        self._context = creation_context
        self._web = requests
        self.debug = creation_context.get('debug', False)

    @property
    def web(self):
        'Gateway to access web, mostly used by decorators (maybe).'
        return self._web

    def _full_context(self, incoming):
        '''
        Creates full context for command to be executed.
        '''
        result = self._context.copy()
        result.update(incoming)
        return result

    def _ensure_https(self, url):
        link = url.strip()
        if link.startswith('https://'):
            return link
        form = 'Security violation: URL %s does not start with https://'
        raise ConnectionError(form % link)

    def _ensure_api_endpoint(self, url):
        link = url.strip()
        if link.startswith('https://api.'):
            return link
        form = 'Contract violation: URL %s is not API endpoint.'
        raise ConnectionError(form % link)
