'''
This module provides authorize command to get authorization token from cloud.
'''

import json
import sys
from .abstract_ import CommandBase, everyparser


RUNTIME_CAPS = dict(secret=dict(read=True),
                    livedata=dict(read=True, write=True),
                    workitemdata=dict(read=True, write=True),
                    artifact=dict(write=True))


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by authorize command.
    '''
    argbloks.default_minutes_argument(parser)
    argbloks.mandatory_workspace_argument(parser)
    argbloks.optional_silent_flag(parser)
    return everyparser(parser, argbloks)


class Command(CommandBase):
    '''
    Authorize converts API key to valid authorization JWT token.
    '''
    def __init__(self, creation_context):
        super(Command, self).__init__(creation_context)
        minutes = creation_context.get('minutes', 1)
        caps = dict(expiresIn=60 * minutes,
                    capabilities=RUNTIME_CAPS.copy())
        self.authorization_capability = caps

    def __call__(self, context):
        '''
        Actual executed command. This method is actual authorize structure.
        '''
        local_context = self._full_context(context)
        silent = local_context.get('silent', False)
        reply = local_context.get('@authorization', None)
        if reply and not silent:
            json.dump(reply, sys.stdout, indent=2)
        return True
