'''
This module provides base class forn *load commands.
'''

from robocode.authorization import AUTHORIZATION_HEADER
from .abstract_ import CommandBase, everyparser

API_FORMAT = '%(endpoint)s/activity-v1/workspaces/%(workspace)d' \
        '/activities/%(activity)d/package/%(direction)slink'


def anyloadparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by *load command.
    '''
    argbloks.optional_force_flag(parser)
    argbloks.default_package_argument(parser)
    argbloks.mandatory_workspace_argument(parser)
    argbloks.mandatory_activity_argument(parser)
    return everyparser(parser, argbloks)


class AnyloadBase(CommandBase):
    'Base class for Download and Upload commands'

    def _validate_task_package(self, force, filename):
        raise NotImplementedError()

    def _do_task(self, url, filename):
        raise NotImplementedError()

    def _common_load(self, direction, context):
        local_context = self._full_context(context)
        force = local_context.get('force', False)
        filename = local_context.get('package', None)
        workspace = local_context.get('workspace', None)
        activity = local_context.get('activity', None)
        authorization = local_context.get('authorization', None)
        endpoint = local_context.get('endpoint', None)
        self._validate_common_options(endpoint, authorization)
        self._validate_task_package(force, filename)
        url = self._activity_url(direction, endpoint, workspace, activity)
        link = self._on_get_error(direction,
                                  *self._get_link(url, authorization))
        return self._on_task_error(direction, self._do_task(link, filename))

    def _validate_common_options(self, endpoint, authorization):
        if not endpoint:
            raise ValueError('Value for endpoint is missing (authorization).')
        if not authorization:
            raise ValueError('Value for authorization is missing.')
        return True

    def _activity_url(self, direction, endpoint, workspace, activity):
        # pylint: disable=unused-argument
        # pylint: disable=too-many-arguments
        endpoint = endpoint.rstrip('/')
        result = API_FORMAT % locals()
        return result.strip()

    def _get_link(self, url, authorization):
        headers = {AUTHORIZATION_HEADER: authorization}
        https = self._ensure_https(url)
        response = self._web.get(https, headers=headers)
        link = None
        if response.status_code == 200:
            data = response.json()
            link = data.get('uri', None)
        return response.status_code, link

    def _on_get_error(self, label, status, link):
        message = None
        if status == 401:
            message = "Unauthorized, check your credentials!"
        elif status == 403:
            message = "Forbidden, check your credentials and workspace!"
        elif status == 404:
            message = "Could not find workspace/activity."
        elif status >= 500:
            message = 'Something went really wrong.'
        elif not link:
            message = "Robocloud did not return %s link." % label
        if message:
            raise ConnectionError("%s [status code: %d]" % (message, status))
        return link

    def _on_task_error(self, direction, status_code):
        if status_code != 200:
            template = 'Could not %s package. [status code: %d]'
            raise ConnectionError(template % (direction, status_code))
        return True
