'''
This module provides download command to fetch package from Robocloud.
'''

import pathlib
from .baseload_ import AnyloadBase, anyloadparser


PACKAGE_CAPS = dict(package=dict(read=True))
DOWNLOAD_CAPS = dict(expiresIn=120, capabilities=PACKAGE_CAPS)


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by download command.
    '''
    return anyloadparser(parser, argbloks)


class Command(AnyloadBase):
    '''
    Download command fetches existing package from Robocloud.
    '''

    authorization_capability = DOWNLOAD_CAPS.copy()

    def __call__(self, context):
        return self._common_load('download', context)

    def _validate_task_package(self, force, filename):
        if not filename:
            raise ValueError('Package file name is missing (--package).')
        fullpath = pathlib.Path(filename).expanduser().absolute()
        if not force and fullpath.exists():
            note = 'Package already exists: %s' % fullpath
            raise FileExistsError(note)
        return True

    def _do_task(self, url, filename):
        https = self._ensure_https(url)
        response = self._web.get(https, stream=True)
        if response.status_code == 200:
            with open(filename, 'wb+') as sink:
                for chunk in response.iter_content():
                    sink.write(chunk)
        return response.status_code
