'''
This module provides fix command to help user to fix automatically fixable
things inside package.
'''

import pathlib
from .abstract_ import CommandBase, everyparser
from .tooling_ import prepare_executables, find_all_entrypoints


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by fix command.

    What does it do then?

    Fix one: make files in bin/ and entrypoints/ directories executable.
    '''
    argbloks.default_directory_argument(parser)
    argbloks.optional_dryrun_flag(parser)
    argbloks.optional_silent_flag(parser)
    return everyparser(parser, argbloks)


def quickfix(directory):
    '''
    To fix given directory.
    '''
    fix = Command(dict(directory=directory, silent=True))
    return fix(dict())


class Command(CommandBase):
    '''
    Fix command provides automation to fix known issues inside packages.
    '''
    def __call__(self, context):
        '''
        Actual executed command. This method is actual fix structure.
        '''
        local_context = self._full_context(context)
        directory = pathlib.Path(local_context.get('directory', '.'))
        dryrun = local_context.get('dryrun', False)
        silent = local_context.get('silent', False)
        self._validate_directory(directory)
        self._fix_executables(directory, silent, dryrun)
        return True

    def _validate_directory(self, directory):
        if not directory.is_dir():
            raise NotADirectoryError('Invalid directory: %s' % directory)
        return True

    def _fix_executables(self, directory, silent, dryrun):
        if dryrun or not silent:
            print('Fixing executable files:')
        for entrypoints in find_all_entrypoints(directory):
            prepare_executables(entrypoints.parent, not silent, dryrun)
