'''
This module provides upload command to push package into Robocloud.
'''

import pathlib
from .baseload_ import AnyloadBase, anyloadparser


PACKAGE_CAPS = dict(package=dict(write=True))
UPLOAD_CAPS = dict(expiresIn=120, capabilities=PACKAGE_CAPS)


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by upload command.
    '''
    return anyloadparser(parser, argbloks)


class Command(AnyloadBase):
    '''
    Upload command pushes existing package into Robocloud.
    '''

    authorization_capability = UPLOAD_CAPS.copy()

    def __call__(self, context):
        return self._common_load('upload', context)

    def _validate_task_package(self, force, filename):
        if not filename:
            raise ValueError('Package file name is missing (--package).')
        fullpath = pathlib.Path(filename).expanduser().absolute()
        if not fullpath.is_file():
            note = 'Cannot see package: %s' % fullpath
            raise FileNotFoundError(note)
        return True

    def _do_task(self, url, filename):
        https = self._ensure_https(url)
        with open(filename, 'rb') as stream:
            response = self._web.put(https, data=stream)
            return response.status_code
