'''
This module is shared collection of validators for inputs needed by various
commands in Robocode CLI.
'''

import pathlib


def validate_optional_variables_argument(variables):
    'Validation operations needed for optional --variables argument.'
    if variables and not pathlib.Path(variables).is_file():
        raise FileNotFoundError('Missing variables file: %s' % variables)
    return True


WORKAREA_NOT_DIRECTORY = '''
Workarea is not a directory: %s (use --workarea argument to give existing
directory).
'''.strip()


def validate_mandatory_workarea_argument(workarea):
    'Validation operations needed for mandatory --workarea argument.'
    if not workarea:
        raise ValueError('Missing workarea name (--workarea).')
    if not pathlib.Path(workarea).is_dir():
        raise NotADirectoryError(WORKAREA_NOT_DIRECTORY % workarea)
    return True
